﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;

namespace Common
{
    public static class Extensions
    {
        // Not an extension to avoid confusion with example
        public static bool IsWeekDay(DateTime timeOfToll)
            => timeOfToll.DayOfWeek switch
            {
                DayOfWeek.Saturday => false,
                DayOfWeek.Sunday => false,
                _ => true,
            };

        public static IResult<T> Start<T>(this T startValue)
            => Result.Success<T>(startValue);

        public static IResult<T> IfNotFailed<TIn, T>(this IResult<TIn> currentResult,
                Func<ISuccessResult<TIn>, IResult<T>> operation)
        {
            if (currentResult is ISuccessResult<TIn> successResult)
            {
                var newResult = operation(successResult);
                // This is messy, intend to clean up 
                newResult.AddPreviousResult(currentResult.Context.PreviousResults.Append(currentResult) );
                return newResult;
            }
            return Result.Fail<T>(currentResult);
        }

        public static IResult<TIn> ReportFailure<TIn>(this IResult<TIn> currentResult)
        {
            if (currentResult is IFailResult)
            {
                Logger.LogError(currentResult.Message);
            }
            return currentResult;
        }

        public static (bool found, T value) TryGetPreviousValue<T>(this IResult result)
            => result is ISuccessResult<T> foundResult
                   ? (true, foundResult.Value)
                   : result is ISuccessResult<object> objectResult
                                 && objectResult.Value is T value
                           ? (true, value)
                           : (false, default);

        private static readonly Dictionary<object, (string Name, object FuncAsObject)> cache
            = new Dictionary<object, (string Name, object FuncAsObject)>();
        public static (string name, Func<TParam, TReturn> operation) GetNameAndFunc<TParam, TReturn>(
            this Expression<Func<TParam, TReturn>> operationExpression)
        {
            if (cache.TryGetValue(operationExpression, out (string Name, object FuncAsObject) tuple))
            {
                // The following needs work because it currently returns null without warning. 
                return (tuple.Name, tuple.FuncAsObject as Func<TParam, TReturn>);
            }

            Func<TParam, TReturn> func = operationExpression.Compile();
            Expression body = operationExpression.Body;
            var name = "Unknown operation";
            switch (body)
            {
                case MethodCallExpression methodCallExpression:
                    name = methodCallExpression.Method.Name;
                    break;

            }
            cache[operationExpression] = (name, func);
            return (name, func);
        }

        private static void RecordComplete(string operationName)
            => Logger.LogInfo($"{operationName} complete");
        public static void RecordIssue<T>(Result<T> result, string step)
            => Logger.Log(result.Message, Severity.Error);
    }
}
