﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Common.Find
{
    public abstract class FindResult<TLookFor, TFind> : Result<TFind>, IFindResult<TLookFor>
    {
        protected FindResult(TLookFor lookFor, IResult<TFind> result, string message)
            : base(message)
        {
            LookFor = lookFor;
            Result = result;
        }

        public TLookFor LookFor { get; }
        public IResult<TFind> Result { get; }

        public class Success : FindResult<TLookFor, TFind>, IResult<TFind>, ISuccessResult<TFind>
        {
            public Success(TLookFor lookFor, ISuccessResult<TFind> result, string message = null)
                : base(lookFor, result, message)
            { }
            public TFind Value
                => ((ISuccessResult<TFind>)Result).Value;
        }

        public class Fail : FindResult<TLookFor, TFind>, IResult<TFind>, IFailResult
        {
            public Fail(TLookFor lookFor, IResult<TFind> result, string message = null)
                : base(lookFor, result, message) { }
        }
    }

    public static class FindExtensions
    {
        public static FindResult<TLookFor, TFind> LookFor<TLookFor, TFind>(this TLookFor lookFor)
            => new FindResult<TLookFor, TFind>.Fail(
                        lookFor, Result.Fail<TFind>());

        public static FindResult<TLookFor, TFind> IfNotFound<TLookFor, TFind>(
                this FindResult<TLookFor, TFind> previous,
                Func<TLookFor, IResult<TFind>> lookForFunc)
        {
            if (previous is ISuccessResult<TFind>)
            {
                return previous;
            }
            return lookForFunc(previous.LookFor) is ISuccessResult<TFind> successResult
                ? new FindResult<TLookFor, TFind>.Success(previous.LookFor, successResult)
                : previous;
        }

        public static IResult<TFind> GetValueResult<TLookFor, TFind>(
                    this FindResult<TLookFor, TFind> previous)
            => previous is FindResult<TLookFor, TFind>.Success successResult
                ? successResult.Result
                : (IResult<TFind>)Result.Fail<TFind>("Value not found");
    }
}
