﻿using CommercialRegistration;
using Common;
using ConsumerVehicleRegistration;
using LiveryRegistration;
using System;
using System.Linq;

namespace ExternalSystem
{

    public static class BillingSystem
    {
        public static IResult<Guid> SendBill(
            this ISuccessResult<(decimal toll, object vehicle)> successResult)
        {
            var toll = successResult.Value.toll;
            if (toll < 0)
            {
                return Result.Fail<Guid>("very oops");
            }

            // TODO: For practice, change this to the kind of find in GetVehicleRegistration
            // Also note that you need to thread the original Result (passed to Find) through 
            // because any rewrite of that Result will change the Type arg to object.
            var registrationResult = FindRegistration<CarRegistration>(successResult, SendCarBill);
            if (registrationResult is IFailResult)
            {
                registrationResult = FindRegistration<DeliveryTruckRegistration>(successResult, SendTruckBill);
            }
            if (registrationResult is IFailResult)
            {
                registrationResult = FindRegistration<BusRegistration>(successResult, SendBusBill);
            }
            if (registrationResult is IFailResult)
            {
                registrationResult = FindRegistration<TaxiRegistration>(successResult, SendTaxiBill);
            }
            return registrationResult;
        }

        private static IResult<Guid> FindRegistration<T>(
            ISuccessResult<(decimal toll, object vehicle)> successResult,
            Func<decimal, T, IResult<Guid>> sendBillFunc)
            where T : class
        {
            var x = successResult.Context.PreviousResults
                        .OfType<IResult<T>>()
                        .SingleOrDefault();
            return x is ISuccessResult<T> findSuccessResult
                ? sendBillFunc(successResult.Value.toll, findSuccessResult.Value)
                : Result.Fail<Guid>(successResult);
        }


        // The following methods are stubbed until the links to these systems is created
        private static IResult<Guid> SendTruckBill(decimal toll, DeliveryTruckRegistration truck)
        {
            Logger.LogInfo($"Truck ({truck.LicensePlate}) - {toll}");
            return Result.Success<Guid>(Guid.NewGuid());
        }

        private static IResult<Guid> SendBusBill(decimal toll, BusRegistration bus)
        {
            Logger.LogInfo($"Bus ({bus.LicensePlate}) - {toll}");
            return Result.Success<Guid>(Guid.NewGuid());
        }

        private static IResult<Guid> SendTaxiBill(decimal toll, TaxiRegistration taxi)
        {
            Logger.LogInfo($"Taxi ({taxi.LicensePlate}) - {toll}");
            return Result.Success<Guid>(Guid.NewGuid());
        }

        private static IResult<Guid> SendCarBill(decimal toll, CarRegistration car)
        {
            Logger.LogInfo($"Car ({car.LicensePlate}) - {toll}");
            return Result.Success<Guid>(Guid.NewGuid());
        }
    }
}
